@@ -1,36 +1,35 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from socket import gethostname
 import logging
 import os
-from typing import Optional
 from distutils.util import strtobool
+from socket import gethostname
+from typing import Optional
 
-from opentelemetry.sdk.resources import SERVICE_NAME, Resource, SERVICE_VERSION, Attributes
+import pkg_resources
+import sentry_sdk
+from opentelemetry import trace
 from opentelemetry.instrumentation.fastapi import FastAPIInstrumentor
+from opentelemetry.sdk.resources import SERVICE_NAME, Resource, SERVICE_VERSION, Attributes
 from opentelemetry.sdk.trace import TracerProvider, Span
 from opentelemetry.sdk.trace.export import BatchSpanProcessor, ConsoleSpanExporter
-from opentelemetry import trace
-
 from opentelemetry.sdk.trace.sampling import ALWAYS_ON
 
-from fastapialchemycollector.utils.once import Once
-from fastapialchemycollector.version import __version__
-from fastapialchemycollector.consts import (
-    METIS_REQUEST_SPAN_ATTRIBUTE_IDENTIFIER,
+from common.consts import (
+    METIS_REQUEST_SPAN_ATTRIBUTE_IDENTIFIER, SENTRY_DSN,
 )
-from fastapialchemycollector.exporters.file_exporter import MetisFileExporter
-from fastapialchemycollector.exporters.remote_exporter import MetisRemoteExporter
-from fastapialchemycollector.plan_collect_type import PlanCollectType
-from fastapialchemycollector.utils.env_var import extract_additional_tags_from_env_var
-from fastapialchemycollector.utils.singleton_class import SingletonMeta
+from common.exporters.remote_exporter import MetisRemoteExporter
+from common.plan_collect_type import PlanCollectType
+from common.utils.env_var import extract_additional_tags_from_env_var
+from common.utils.log import log
+from common.utils.once import Once
+from common.utils.singleton_class import SingletonMeta
+from common.version import __version__
 
 METIS_INSTRUMENTATION_STR = "METIS_INSTRUMENTATION"
 
 logger = logging.getLogger(__name__)
 
-FILE_NAME = "metis-log-collector.log"
-
 os.environ[
     "OTEL_INSTRUMENTATION_HTTP_CAPTURE_HEADERS_SERVER_REQUEST"
 ] = "content-type,custom_request_header"
@@ -38,14 +37,8 @@ os.environ[
     "OTEL_INSTRUMENTATION_HTTP_CAPTURE_HEADERS_SERVER_RESPONSE"
 ] = "content-type,content-length,custom_request_header"
 
-EXPLAIN_SUPPORTED_STATEMENTS = (
-    "SELECT",
-    "INSERT",
-    "UPDATE",
-    "DELETE",
-)
-
 
+@log
 def add_quote_to_value_of_type_string(value):
     if isinstance(value, str):
         new_value = str(value).replace("'", "''")
@@ -53,18 +46,7 @@ def add_quote_to_value_of_type_string(value):
     return value
 
 
-def fix_sql_query(sql, params):
-    """without the fix the query is not working because string is not quoted"""
-    fixed_param = params
-    if isinstance(params, dict):
-        fixed_param = {
-            key: add_quote_to_value_of_type_string(value)
-            for key, value in params.items()
-        }
-
-    return sql % fixed_param
-
-
+@log
 def _normalize_vendor(vendor):
     """Return a canonical name for a type of database."""
     if not vendor:
@@ -79,13 +61,12 @@ def _normalize_vendor(vendor):
     return vendor
 
 
-def _convert_items_to_metis_tags(tags_dict: Optional[Attributes]):
-    return {f'app.tag.{key}': val for key, val in tags_dict.items()}
-
+@log
 def _build_resource(
         service_name: str,
         service_version: str,
         resource_tags: Attributes,
+        sentry_logger: Optional[sentry_sdk.Hub],
 ) -> Resource:
     attrs = {}
 
@@ -97,7 +78,9 @@ def _build_resource(
     try:
         attrs["host.name"] = gethostname()
     except Exception as e:
-        logger.exception("Cloudnt get hostname for service")
+        if sentry_logger is not None:
+            sentry_logger.capture_exception(e)
+    attrs['metis.sdk.version'] = pkg_resources.get_distribution('fastapialchemycollector').version
 
     if resource_tags:
         attrs.update(_convert_items_to_metis_tags(resource_tags))
@@ -106,33 +89,53 @@ def _build_resource(
     if len(metis_tags_env_vars) > 0:
         attrs.update(_convert_items_to_metis_tags(metis_tags_env_vars))
 
-
     return Resource.create(attrs)
 
 
+@log
+def _convert_items_to_metis_tags(tags_dict: Optional[Attributes]):
+    return {f'app.tag.{key}': val for key, val in tags_dict.items()}
+
+
+@log
 def setup(
         service_name: str,
+        api_key: str,
         service_version: Optional[str] = None,
         resource_tags: Optional[Attributes] = None,
         plan_collection_option: Optional[PlanCollectType] = PlanCollectType.ESTIMATED,
-        file_name: Optional[str] = None,
-        api_key: Optional[str] = None,
-        dsn: Optional[str] = None):
+        dsn: Optional[str] = "https://ingest.metisdata.io/",
+        report_errors: bool = True):
     _is_metis_instrumentation_enable = strtobool(os.getenv(METIS_INSTRUMENTATION_STR, 'true'))
     if not _is_metis_instrumentation_enable:
         logging.debug("Metis instrumentation is disabled")
         return
 
+    if api_key and report_errors:
+        sentry_logger = sentry_sdk.Hub(sentry_sdk.Client(
+            dsn=SENTRY_DSN,
+            traces_sample_rate=1.0,
+            with_locals=False,
+            request_bodies="never",
+            auto_session_tracking=False,
+            release=__version__,
+        ))
+        sentry_logger.scope.set_user({'id': api_key})
+    else:
+        sentry_logger = None
+
     metis_interceptor = MetisInstrumentor(service_name,
                                           service_version=service_version,
                                           resource_tags=resource_tags,
-                                          plan_collection_option=plan_collection_option)
+                                          plan_collection_option=plan_collection_option,
+                                          sentry_logger=sentry_logger)
 
-    metis_interceptor.set_exporters(file_name, api_key, dsn)
+    metis_interceptor.set_exporters(api_key, dsn)
 
     return metis_interceptor
 
 
+@log
 def shutdown():
     trace_provider = trace.get_tracer_provider()
 
@@ -142,19 +145,22 @@ def shutdown():
 
 # pylint: disable=too-few-public-methods
 class MetisInstrumentor(metaclass=SingletonMeta):
+    @log
     def __init__(self,
                  service_name,
                  service_version: Optional[str] = None,
                  resource_tags: Optional[Attributes] = None,
                  plan_collection_option: Optional[PlanCollectType] = PlanCollectType.ESTIMATED,
+                 sentry_logger: Optional[sentry_sdk.Hub] = None,
                  ):
-        self.api_instrumentor_app = None
+        self.api_app_instance = None
         self.set_exporters_once = Once()
         self.sqlalchemy_instrumentor = None
         self.api_instrumentor = None
         self.plan_collection_option = plan_collection_option
+        self.sentry_logger = sentry_logger
 
-        resource = _build_resource(service_name, service_version, resource_tags)
+        resource = _build_resource(service_name, service_version, resource_tags, sentry_logger)
 
         self.tracer_provider = TracerProvider(sampler=ALWAYS_ON,
                                               resource=resource)
@@ -165,36 +171,35 @@ class MetisInstrumentor(metaclass=SingletonMeta):
             tracer_provider=self.tracer_provider,
         )
 
+    @log
     def set_exporters(self,
-                      file_name: Optional[str] = None,
                       api_key: Optional[str] = None,
                       dsn: Optional[str] = None):
-        is_set = self.set_exporters_once.do_once(self._set_exporters,
-                                                 file_name=file_name, api_key=api_key, dsn=dsn)
+        is_set = self.set_exporters_once.do_once(self._set_exporters, api_key=api_key, dsn=dsn)
 
         if not is_set:
             logger.warning("You've setup metis instrumentation already")
 
+    @log
     def _set_exporters(self,
-                       file_name: Optional[str] = None,
                        api_key: Optional[str] = None,
                        dsn: Optional[str] = None):
-        if not file_name and not dsn:
-            file_name = os.getenv("METIS_LOG_FILE_NAME", FILE_NAME)
-        if file_name:
-            self._add_processor(BatchSpanProcessor(MetisFileExporter(file_name)))
         if bool(dsn) != bool(api_key):
-            raise ValueError("Both endpoint and api_key must be provided")
+            raise ValueError("Endpoint and api_key must be provided")
+
         if dsn is not None:
-            self._add_processor(BatchSpanProcessor(MetisRemoteExporter(dsn, api_key)))
+            self._add_processor(BatchSpanProcessor(MetisRemoteExporter(dsn, api_key, self.sentry_logger)))
 
         if strtobool(os.getenv("DEBUG", 'False')):
             self._add_processor(BatchSpanProcessor(ConsoleSpanExporter()))
 
+    @log
     def _add_processor(self, processor):
         self.tracer_provider.add_span_processor(processor)
 
+    @log
     def instrument_app(self, app, engine):
+        @log
         def request_hook(
                 span: Span,
                 message: dict,
@@ -204,7 +209,7 @@ class MetisInstrumentor(metaclass=SingletonMeta):
 
         if self.api_instrumentor is None:
             self.api_instrumentor = FastAPIInstrumentor()
-            self.api_instrumentor_app = app
+            self.api_app_instance = app
 
         self.api_instrumentor.instrument_app(
             app,
@@ -213,11 +218,13 @@ class MetisInstrumentor(metaclass=SingletonMeta):
             excluded_urls='favicon'
         )
 
-        engine = engine.sync_engine if hasattr(engine, 'sync_engine') and engine.sync_engine is not None else engine
-        from fastapialchemycollector.alchemy_instrumentation import MetisSQLAlchemyInstrumentor
+        if hasattr(engine, 'sync_engine') and engine.sync_engine is not None:
+            engine = engine.sync_engine
+        from common.alchemy_instrumentation import MetisSQLAlchemyInstrumentor
 
         if self.sqlalchemy_instrumentor is None:
             self.sqlalchemy_instrumentor = MetisSQLAlchemyInstrumentor()
+            self.sqlalchemy_instrumentor.sentry_logger = self.sentry_logger
 
         self.sqlalchemy_instrumentor.instrument(
             engine=engine,
@@ -225,11 +232,12 @@ class MetisInstrumentor(metaclass=SingletonMeta):
             trace_provider=self.tracer_provider,
         )
 
+    @log
     def uninstrument_app(self):
-        if self.api_instrumentor is not None:
-            FastAPIInstrumentor.uninstrument_app(self.api_instrumentor_app)
+        if self.api_instrumentor is not None and self.api_app_instance is not None:
+            FastAPIInstrumentor.uninstrument_app(self.api_app_instance)
             self.api_instrumentor.uninstrument()
-            self.api_instrumentor_app = None
+            self.api_app_instance = None
 
         if self.sqlalchemy_instrumentor is not None:
             self.sqlalchemy_instrumentor.uninstrument()
